package com.buzzvil.buzz_booster

import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.*
import com.buzzvil.booster.external.BuzzBooster
import com.buzzvil.booster.external.BuzzBoosterUser
import com.buzzvil.booster.external.BuzzBoosterConfig
import com.buzzvil.booster.external.campaign.*
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.BroadcastReceiver
import java.lang.Object
import android.util.Log

/** BuzzBoosterPlugin */
class BuzzBoosterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, EventChannel.StreamHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var context: Context
    private var eventSink: EventChannel.EventSink? = null
    private var activity: Activity? = null
    private var linkUrl: String? = null
    private var optInMarketingMoveButtonClicked: Boolean = false

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "buzz_booster")
        channel.setMethodCallHandler(this)

        val linkEventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "buzz_booster/link")
        linkEventChannel.setStreamHandler(this)
        handleUrl()
        handleOptInMarketingCampaignMoveButtonClicked()

        flutterPluginBinding
            .platformViewRegistry
            .registerViewFactory(
                "CampaignFloatingActionButton",
                FLCampaignFloatingActionButtonFactory()
            )

        flutterPluginBinding
            .platformViewRegistry
            .registerViewFactory("CampaignEntryView", FLCampaignEntryViewFactory())
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "init") {
            val appKey = call.argument("androidAppKey") as String?
            if (appKey != null) {
                val config = BuzzBoosterConfig(appKey!!)
                BuzzBooster.init(context, config)
                BuzzBooster.getInstance().setCustomCampaignActionButtonClickListener(object: CustomCampaignActionButtonClickListener {
                    override fun onClick(url: String) {
                        BuzzBooster.getInstance().finishActivity(context, BuzzBoosterActivityTag.CAMPAIGN_DETAIL)
                        BuzzBooster.getInstance().finishActivity(context, BuzzBoosterActivityTag.CAMPAIGN_LIST)
                        linkUrl = url
                        handleUrl()
                    }
                })
                BuzzBooster.getInstance().setOptInMarketingCampaignMoveButtonClickListener(object: OptInMarketingCampaignMoveButtonClickListener {
                    override fun onClick() {
                        BuzzBooster.getInstance().finishActivity(context, BuzzBoosterActivityTag.CAMPAIGN_DETAIL)
                        BuzzBooster.getInstance().finishActivity(context, BuzzBoosterActivityTag.CAMPAIGN_LIST)
                        optInMarketingMoveButtonClicked = true
                        handleOptInMarketingCampaignMoveButtonClicked()
                    }
                })
                result.success(null)
            } else {
                result.error("400", "AndroidAppKey is required", null)
            }
        } else if (call.method == "setUser") {
            val userId = call.argument("userId") as String?
            val optInMarketing = call.argument("optInMarketing") as Boolean?
            val properties = call.argument("properties") as Map<String, String>?
            
            if (userId != null) {
                val userBuilder = BuzzBoosterUser.Builder()
                    .setUserId(userId)
                
                if (optInMarketing != null){
                    userBuilder.setOptInMarketing(optInMarketing)
                }
                if (properties != null) {
                    properties.forEach { entry -> 
                        userBuilder.setProperty(entry.key, entry.value)
                    }
                }
                BuzzBooster.setUser(userBuilder.build())
            } else {
                BuzzBooster.setUser(null)
            }

            result.success(null)
        } else if (call.method == "sendEvent") {
            val eventName = call.argument("eventName") as String?
            if (eventName != null) {
                val eventValues = call.argument("eventValues") as Map<String, String>?
                if (eventValues != null) {
                    BuzzBooster.getInstance().sendEvent(eventName, eventValues)
                } else {
                    BuzzBooster.getInstance().sendEvent(eventName)
                }
                result.success(null)
            } else {
                result.error("400", "EventName is required", null)
            }
        } else if (call.method == "showInAppMessage") {
            if (activity != null) {
                BuzzBooster.getInstance().showInAppMessage(activity!!)
            }
            result.success(null)
        } else if (call.method == "showCampaign") {
            if (activity != null) {
                BuzzBooster.getInstance().showCampaign(activity!!)
            }
            result.success(null)
        } else if (call.method == "showCampaignWithId") {
            if (activity != null) {
                val campaignId = call.argument("campaignId") as String?
                BuzzBooster.getInstance().showCampaign(activity!!, campaignId!!)
            }
            result.success(null)
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivity() {
        activity = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onListen(arguments: Any?, eventSink: EventChannel.EventSink?) {
        this.eventSink = eventSink
    }
    override fun onCancel(arguments: Any?) { 
        this.eventSink = null
    }

    private fun handleUrl() {
        if (linkUrl != null) {
            if (eventSink != null) {
                eventSink!!.success(mapOf(
                    "event" to "customCampaignMoveButtonClicked",
                    "url" to linkUrl
                ))
                linkUrl = null
            }
        }
    }

    private fun handleOptInMarketingCampaignMoveButtonClicked() {
        if (optInMarketingMoveButtonClicked) {
            if (eventSink != null) {
                eventSink!!.success(mapOf(
                    "event" to "OptInMarketingCampaignMoveButtonClicked"
                ))
                optInMarketingMoveButtonClicked = false
            }
        }
    }
}
