#import "BuzzBoosterPlugin.h"
#import "FLCampaignFloatingActionButtonFactory+View.h"
#import "FLCampaignEntryViewFactory+View.h"

static NSString *const kEventsChannel = @"buzz_booster/link";

@interface BuzzBoosterPlugin () <FlutterStreamHandler>

@end

@implementation BuzzBoosterPlugin {
  FlutterEventSink _eventSink;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"buzz_booster"
            binaryMessenger:[registrar messenger]];
  BuzzBoosterPlugin* instance = [[BuzzBoosterPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];

  FlutterEventChannel *chargingChannel = [FlutterEventChannel eventChannelWithName:kEventsChannel
                                                                   binaryMessenger:[registrar messenger]];
  [chargingChannel setStreamHandler:instance];

  FLCampaignFloatingActionButtonFactory* campaignFloatingActionButtonFactory =
      [[FLCampaignFloatingActionButtonFactory alloc] initWithMessenger:registrar.messenger];
  [registrar registerViewFactory:campaignFloatingActionButtonFactory withId:@"CampaignFloatingActionButton"];
  FLCampaignEntryViewFactory* campaignEntryViewFactory =
      [[FLCampaignEntryViewFactory alloc] initWithMessenger:registrar.messenger];
  [registrar registerViewFactory:campaignEntryViewFactory withId:@"CampaignEntryView"];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"init" isEqualToString:call.method]) {
    NSString *appKey = call.arguments[@"iosAppKey"];
    if (appKey != nil && ![appKey isKindOfClass:[NSNull class]]) {
      BSTConfig *config = [BSTConfig configWithBlock:^(BSTConfigBuilder * _Nonnull builder) {
        builder.appKey = appKey;
      }];
      [BuzzBooster initializeWithConfig:config];
      [BuzzBooster setCustomCampaignDelegate:self];
      [BuzzBooster setOptInMarketingCampaignDelegate:self];
      result(nil);
    } else {
      FlutterError *error = [FlutterError errorWithCode:@"400" message:@"iOSAppKey is required" details:nil];
      result(error);
    }
  } else if ([@"startService" isEqualToString:call.method]){
    [BuzzBooster startService];
    result(nil);
  } else if ([@"setUser" isEqualToString:call.method]) {
    NSString * _Nullable userId = call.arguments[@"userId"];
    NSNumber * _Nullable optInMarketing = call.arguments[@"optInMarketing"];
    NSDictionary * _Nullable properties = call.arguments[@"properties"];
    if (userId != nil && ![userId isKindOfClass:[NSNull class]]) {
      BSTUser *user = [BSTUser userWithBlock:^(BSTUserBuilder * _Nonnull builder) {
        builder.userId = userId;
        builder.properties = properties;
        if (optInMarketing != nil && ![optInMarketing isKindOfClass:[NSNull class]]) {
          if ([optInMarketing boolValue]) {
            builder.marketingStatus = BSTMarketingStatusOptIn;
          } else {
            builder.marketingStatus = BSTMarketingStatusOptOut;
          }
        } else {
          builder.marketingStatus = BSTMarketingStatusUndetermined;
        }
      }];
      [BuzzBooster setUser:user];
    } else {
      [BuzzBooster setUser:nil];
    }
    result(nil);
  } else if ([@"sendEvent" isEqualToString:call.method]){
    NSString *eventName = call.arguments[@"eventName"];
    NSDictionary * _Nullable eventValues = call.arguments[@"eventValues"];
    if (eventValues != nil && ![eventValues isKindOfClass:[NSNull class]]) {
      [BuzzBooster sendEventWithEventName:eventName
                              eventValues:eventValues];
    } else {
      [BuzzBooster sendEventWithEventName:eventName];
    }
    result(nil);
  } else if ([@"showInAppMessage" isEqualToString:call.method]){
    UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
    [BuzzBooster showInAppMessageWithViewController:viewController];
    result(nil);
  } else if ([@"showCampaign" isEqualToString:call.method]){
    UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
    [BuzzBooster showCampaignWithViewController:viewController];
    result(nil);
  } else if ([@"showCampaignWithId" isEqualToString:call.method]){
    UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
    NSString *campaignId = call.arguments[@"campaignId"];
    [BuzzBooster showCampaignWithViewController:viewController campaignId:campaignId];
    result(nil);
  } else {
    result(FlutterMethodNotImplemented);
  }
}

#pragma mark --FlutterStreamHandler
- (FlutterError *_Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:(nonnull FlutterEventSink)eventSink {
  _eventSink = eventSink;
  return nil;
}

- (FlutterError *_Nullable)onCancelWithArguments:(id _Nullable)arguments {
  _eventSink = nil;
  return nil;
}

#pragma mark --BSTCustomCampaignDelegate
- (void)onMoveButtonClickedWithUrl:(NSString *)url {
  UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
  [viewController dismissViewControllerAnimated:NO completion:nil];
  if (_eventSink) {
    _eventSink(@{
      @"event": @"customCampaignMoveButtonClicked",
      @"url": url
    });
  }
}

#pragma mark --BSTOptInMarketingCampaignDelegate
- (void)onMoveButtonClicked {
  UIViewController *viewController = [UIApplication sharedApplication].delegate.window.rootViewController;
  [viewController dismissViewControllerAnimated:NO completion:nil];
  if (_eventSink) {
    _eventSink(@{
      @"event": @"OptInMarketingCampaignMoveButtonClicked",
    });
  }
}

@end
